import {
  Component,
  OnInit,
  AfterViewInit,
  ViewChild,
  ChangeDetectorRef,
} from '@angular/core';

import { finalize } from 'rxjs/operators';
import { BsModalService } from 'ngx-bootstrap';
import { BsLocaleService } from 'ngx-bootstrap/datepicker';
import { DatatableComponent } from '@swimlane/ngx-datatable/release/components';

import { Logger } from '@core/logger.service';
import { Dialogs } from '@core/dialogs.service';
import { LoggerFactory } from '@core/logger-factory.service';
import { OrdersService } from '../../shared/orders.service';

import { NgxDataTableDirective } from '@app/shared/directives/ngx-datatable.directive';

import { EditOrderModalComponent } from '../edit-order-modal/edit-order-modal.component';
import { Order } from '../../shared/models';
import { QueryPageListEvent, QueryTemplate, RowDetailToggleEvent } from '@app/shared/models/type';

@Component({
  selector: 'app-orders',
  templateUrl: './orders.component.html',
  styleUrls: ['./orders.component.scss'],
})
export class OrdersComponent implements OnInit, AfterViewInit {
  log: Logger;
  loading = false;
  orders: Array<Order>;
  selectedOrders: Array<Order> = [];
  notHideColumns: string[] = ['price'];
  queryTemplates: QueryTemplate[] = [
    {
      name: 'Default',
      template: {
        op: 'or',
        rules: [
          { field: 'subject', op: 'cn', data: '' },
          { field: 'Date', op: 'bt' },
          { field: 'Amount', op: 'eq' },
        ],
        groups: [],
      },
    },
  ];

  subjectName: string = '订单摘要<br/>订单摘要1';

  @ViewChild('appNgxDataTable') ngxDataTable: NgxDataTableDirective;
  @ViewChild('dt') datatable: DatatableComponent;

  constructor(
    private dialogs: Dialogs,
    private ordersService: OrdersService,
    private modalService: BsModalService,
    private loggerFactory: LoggerFactory,
    private localeService: BsLocaleService,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    this.log = this.loggerFactory.getLogger(`订单列表`);
    this.localeService.use('zh-cn');
  }

  ngOnInit() {}

  ngAfterViewInit() {
    this.changeDetectorRef.detectChanges();
  }

  onSelect(event: any) {
    if (event !== void 0 && event.selected !== void 0) {
      this.selectedOrders = event.selected;
    }
  }

  loadOrders(event: QueryPageListEvent) {
    const params = event.page;
    this.loading = true;
    this.selectedOrders.length = 0;

    this.ordersService
      .getOrders(params)
      .pipe(finalize(() => (this.loading = false)))
      .subscribe(
        (response) => {
          this.datatable.count = response.count;

          this.orders = response.data;
          this.log.debug('订单列表', this.orders);
        },
        (error) => this.log.error('订单获取失败。', error)
      );
  }

  delete(row: Order) {
    this.dialogs.confirm(`真的要删除订单 ${row.subject} 吗？`).subscribe(
      () => {
        this.ordersService.deleteOrder(row).subscribe(
          () => {
            this.ngxDataTable.deleteRefreshData();
            this.log.success(`订单 ${row.subject} 删除成功!`);
          },
          (error) =>
            this.log.error(`订单 ${row.subject} 删除失败,失败信息:`, error)
        );
      },
      () => this.log.debug(`取消删除订单 ${row.subject}`)
    );
  }

  editByModal(row: Order) {
    const initialState = { data: row };
    this.modalService.show(EditOrderModalComponent, {
      initialState,
      ignoreBackdropClick: true,
    });

    const onHidden = this.modalService.onHidden.subscribe((val: Order) => {
      // tslint:disable-next-line:forin
      for (const r in val) {
        row[r] = val[r];
      }
      onHidden.unsubscribe();
    });
  }

  onDetailToggle(event: RowDetailToggleEvent<Order>) {
    console.log('Detail Toggled', event);
  }

  toggleExpandRow(row: Order) {
    console.log('Toggled Expand Row!', row);
    this.datatable.rowDetail.toggleExpandRow(row);
  }
}
