import {
  Directive,
  OnInit,
  AfterViewInit,
  Input,
  Output,
  AfterContentChecked,
  ViewContainerRef,
  EventEmitter,
  ContentChild,
  ChangeDetectorRef,
  OnDestroy
} from '@angular/core';
import { Router } from '@angular/router';

import { DatatableFooterComponent } from 'ngx-datatable-footer';
import { NgxDatatableActionsComponent } from 'ngx-datatable-actions';
import {
  NgxQueryComponent,
  cloneQueryGroup,
  Rule
} from '@zhongruigroup/ngx-query';
import { ColumnMode, SortType } from '@swimlane/ngx-datatable/release/types';
import { DatatableComponent } from '@swimlane/ngx-datatable/release/components';

import { WindowEventListenerService } from '../services/window-event-listener.service';
import {
  QueryPageListEvent,
  QueryPagingSort,
  StoragePaging,
  QueryTerm,
  QueryRule
} from '../models/type';
import { getTermQuery as getQueryTerm } from '../utils/filter';
import { debounceTime } from 'rxjs/operators';
import { merge } from 'rxjs';

@Directive({
  selector: '[appNgxDataTable]',
  exportAs: 'NgxDataTableDirective'
})
export class NgxDataTableDirective
  implements OnInit, AfterViewInit, OnDestroy, AfterContentChecked {
  @Output() data: EventEmitter<QueryPageListEvent> = new EventEmitter();

  @Input() headerHeight = 40;
  @Input() footerHeight = 40;
  @Input() rowHeight = 44;
  @Input() saveState = true;
  /**
   * 缓存状态策略
   * 'local'使用localStorage 'session'使用sessionStorage
   * 默认使用'session'
   */
  @Input() saveStateStrategy: 'local' | 'session' = 'session';
  /**
   * 缓存状态编码
   * 设置为不同的值，用来隔离在同一个页面多次使用本指令产生的缓存状态问题
   */
  @Input() saveStateCode: string = '';
  /**
   * 页面卸载时否清除缓存
   */
  @Input() clearSaveStateOnUnLoad = false;
  @Input() externalPaging = true;
  @Input() ngxQuery: NgxQueryComponent;
  @Input()
  set pageSize(val: number) {
    if (val < 1) {
      throw new Error(`The value '${val}' can't be less than 1.`);
    }

    this._pageSize = val;
    if (this._datatable) {
      this._datatable.limit = this._pageSize;
    }
  }
  get pageSize(): number {
    return this._pageSize;
  }
  @Input()
  set pageIndex(val: number) {
    if (val < 1) {
      throw new Error(`The value '${val}' can't be less than 1.`);
    }
    this._pageIndex = val;
    if (this._datatable) {
      this._datatable.offset = this._pageIndex;
    }
  }
  get pageIndex(): number {
    return this._pageIndex;
  }

  @ContentChild(DatatableFooterComponent) footer: DatatableFooterComponent;
  @ContentChild(NgxDatatableActionsComponent)
  toolbar: NgxDatatableActionsComponent;

  private latestTableWidth: number;

  private _pageSize = 10;
  private _pageIndex = 1;
  private _sorts: any[] = [];
  private _tempQueryTemplates: any;
  private _datatable: DatatableComponent;

  constructor(
    private router: Router,
    private windowEventListenerService: WindowEventListenerService,
    private _view: ViewContainerRef,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit() {
    if (this.saveState === true) {
      this.loadCachePaging();
    }
    this._datatable = (<any>this._view)._data.componentView.component;
    this.initialize(this._datatable, this.ngxQuery);
  }

  ngAfterViewInit() {
    this.emitData();
    const key = this.pagingStateKey();
    if (this.clearSaveStateOnUnLoad) {
      this.windowEventListenerService.addListener(key, 'beforeunload', () => {
        this.clearPagingState();
      });
    }
  }

  ngAfterContentChecked() {
    if (
      this._datatable &&
      this._datatable.element.clientWidth !== this.latestTableWidth
    ) {
      this.latestTableWidth = this._datatable.element.clientWidth;
      this._datatable.recalculate();
      this.changeDetectorRef.detectChanges();
    }
  }

  ngOnDestroy(): void {}

  public refreshData() {
    this.emitData();
  }

  public deleteRefreshData() {
    this.emitData(undefined, undefined, true);
  }

  private initialize(
    datatable: DatatableComponent,
    ngxQuery: NgxQueryComponent
  ): void {
    // datatable
    datatable.headerHeight = this.headerHeight;
    datatable.footerHeight = this.footerHeight;
    datatable.scrollbarV = false;
    datatable.limit = this._pageSize;
    datatable.sortType = SortType.multi;
    datatable.rowHeight = this.rowHeight;
    datatable.offset = this._pageIndex - 1;
    datatable.columnMode = ColumnMode.force;
    datatable.externalPaging = this.externalPaging;

    if (this.footer) {
      this.footer.datatable = datatable;
    }

    if (this._sorts && this._sorts.length > 0) {
      datatable.sorts = this._sorts;
    }

    datatable.messages = {
      emptyMessage: '当前无记录。',
      totalMessage: '行',
      selectedMessage: '选择'
    };

    datatable.page.subscribe((event: any) => {
      if (event.offset || event.offset === 0) {
        this.emitData(event);
      }
    });

    datatable.sort.subscribe((event: any) => {
      this.emitData(undefined, event.sorts);
    });

    // query
    if (ngxQuery) {
      merge(
        ngxQuery.query.pipe(),
        ngxQuery.reset.pipe(debounceTime(50))
      ).subscribe(() => {
        this._datatable.offset = 0;
        this.emitData(undefined, []);
      });

      if (this.saveState === true) {
        ngxQuery.reset.subscribe(() => {
          ngxQuery.queryTemplates = this._tempQueryTemplates;
        });
      }
    }
  }

  private emitData(page?: any, sorts?: any[], deleteRefreshData?: boolean) {
    // page
    if (page === undefined) {
      let offset = this.pageIndex - 1;

      const turnToPreviousPage =
        deleteRefreshData &&
        this._datatable.rows &&
        this._datatable.rows.length === 1;

      if (turnToPreviousPage) {
        offset = this.pageIndex - 2;
      }

      page = {
        limit: this.pageSize,
        offset: offset
      };
    } else {
      this._pageSize = page.limit;
    }

    this._pageIndex = page.offset + 1;

    // sort
    if (sorts === undefined) {
      sorts = this._datatable.sorts;
    } else {
      page.offset = 0;
      this._pageIndex = page.offset + 1;
    }

    const sortArray: QueryPagingSort[] = [];
    for (const sort of sorts) {
      sortArray.push({ field: sort.prop, sort: sort.dir });
    }
    // query
    const queryTerm = getQueryTerm(this.ngxQuery);
    // cache
    if (this.saveState === true) {
      this.cachePagingState(page, sorts, queryTerm);
    }
    // event
    this.data.emit({
      datatable: this._datatable,
      // event: event,
      page: {
        pageSize: page.limit,
        pageIndex: page.offset + 1 || 1,
        sort: sortArray,
        filters: queryTerm.filters,
        filter: queryTerm.query
      },
      query: queryTerm
    });
  }

  /**
   * 缓存分页状态
   */
  private cachePagingState(page?: any, sorts?: any[], query?: QueryTerm) {
    let rules: QueryRule[];

    if (this.ngxQuery) {
      rules = this.ngxQuery.getOriginalQuery().rules;
    } else {
      rules = [];
    }

    const paging: StoragePaging = {
      pageSize: page.limit,
      pageIndex: page.offset + 1 || 1,
      sorts: sorts,
      filters: query.filters,
      rules: rules
    };

    this.savePagingState(paging);
  }

  private loadCachePaging() {
    if (this.ngxQuery) {
      this._tempQueryTemplates = [];
      this.ngxQuery.queryTemplates.forEach((s) => {
        this._tempQueryTemplates.push({
          name: s.name,
          template: cloneQueryGroup(s.template)
        });
      });
    }

    const paging = this.getPagingState();

    if (paging) {
      // page
      this._pageIndex = paging.pageIndex;
      this._pageSize = paging.pageSize;

      if (paging.rules) {
        this.ngxQuery.queryTemplates[0].template.rules = paging.rules as Rule[];
      }

      // sort
      if (paging.sorts && paging.sorts.length > 0) {
        this._sorts = paging.sorts;
      }
    }
  }

  //#region  pagingState 存取

  private savePagingState(value: StoragePaging) {
    const key = this.pagingStateKey();

    const v = JSON.stringify(value);
    if (this.saveStateStrategy === 'local') {
      localStorage.setItem(key, v);
      return;
    }
    if (this.saveStateStrategy === 'session') {
      sessionStorage.setItem(key, v);
      return;
    }
  }

  private getPagingState(): StoragePaging {
    const key = this.pagingStateKey();

    if (this.saveStateStrategy === 'local') {
      return JSON.parse(localStorage.getItem(key));
    }
    if (this.saveStateStrategy === 'session') {
      return JSON.parse(sessionStorage.getItem(key));
    }
  }

  private clearPagingState() {
    const key = this.pagingStateKey();
    if (this.saveStateStrategy === 'local') {
      localStorage.removeItem(key);
    }
    if (this.saveStateStrategy === 'session') {
      sessionStorage.removeItem(key);
    }
  }

  private pagingStateKey(): string {
    return this.router.url + this.saveStateCode + '|saveState|dt';
  }

  //#endregion
}
